<?php
namespace Teknisa\SUP\Modules\Cadastros\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Teknisa\SUP\Util\SupUtilities;

class CadastroCategoriaSupplyTekService {
    public function __construct(SupUtilities $supUtilities) {
        $this->supUtilities = $supUtilities;
    }
    
    /********************
     *      INSERTS     *
     * ******************/
     
    public function insertNewCategory($row) {
        try {
            $query = "INSERT INTO CATEGORIASUPPLY
                        (CDCATEGORIA, NMCATEGORIA, CDDESPPREV, IDATIVO, NRORG)
                    VALUES
                        (:CDCATEGORIA, :NMCATEGORIA, :CDDESPPREV, :IDATIVO, :NRORG)";
                        
            $params = [
                'CDCATEGORIA'       => $row["CDCATEGORIA"],
                'NMCATEGORIA'       => $row['NMDESPPREV'],
                'CDDESPPREV'        => $row['CDDESPPREV'],
                'IDATIVO'           => $row['IDATIVO'],
                'NRORG'             => $row['NRORG']
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
             throw $e;
        }
    }
    
    public function insertNewSubCategory($row) {
        try {
            $query = "INSERT INTO SUBCATEGSUPPLY
                        (CDSUBCATEGORIA, NMSUBCATEGORIA, CDCATEGORIA, IDATIVO, NRORG)
                    VALUES
                        (:CDSUBCATEGORIA, :NMSUBCATEGORIA, :CDCATEGORIA, :IDATIVO, :NRORG)";
                
                    $params = [
                        'CDSUBCATEGORIA'    => $row['CDSUBCATEGORIA'],
                        'NMSUBCATEGORIA'    => $row['NMSUBCATEGORIA'],
                        'CDCATEGORIA'       => $row['CDCATEGORIA'],
                        'IDATIVO'           => $row['IDATIVO'],
                        'NRORG'             => $row['NRORG']
                    ];
                    $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
             throw $e;
        }
    }
    
    public function insertNewProduct($row) {
        try {
            $sql = "INSERT INTO PRODSUPPLY
                    (
                        CDCATEGORIA, 
                        CDSUBCATEGORIA, 
                        CDFORNECED, 
                        NRCOLEPRECO, 
                        CDPRODUTO, 
                        IDATIVO, 
                        NRORG,
                        CDFILCOT,
                        CDREGIAO,
                        CDMARCA
                    )
                    VALUES
                    (
                        :P_CDCATEGORIA,
                        :P_CDSUBCATEGORIA,
                        :P_CDFORNECED,
                        :P_NRCOLEPRECO,
                        :P_CDPRODUTO,
                        :P_IDATIVO,
                        :P_NRORG,
                        :P_CDFILCOT,
                        :P_CDREGIAO,
                        :P_CDMARCA
                    )";
        
            $params = [
                'P_CDCATEGORIA' => $row['CDCATEGORIA'],
                'P_CDSUBCATEGORIA' => $row['CDSUBCATEGORIA'],
                'P_CDFORNECED' => $row['CDFORNECED'],    
                'P_NRCOLEPRECO' => $row['NRCOLEPRECO'],
                'P_CDPRODUTO' => $row['CDPRODUTO'],
                'P_IDATIVO' => $row['IDATIVO'],
                'P_NRORG' => $row['NRORG'],
                'P_CDFILCOT' => $row['CDFILIAL'],
                'P_CDREGIAO' => $row['CDREGIAO'],
                'P_CDMARCA' => $row['CDMARCA']
            ];
            
            $this->supUtilities->crud($sql, $params);
        }
        catch (Exception $e) {
             throw $e;
        }
    }
     
       
    /********************
     *      UPDATES     *
     * ******************/
     
    public function updateSubCategory($row) 
    {
        try {
            $query = "UPDATE SUBCATEGSUPPLY SET NMSUBCATEGORIA = :NMSUBCATEGORIA
                        WHERE CDSUBCATEGORIA = :CDSUBCATEGORIA
                      AND NRORG = :NRORG";
                $params = [
                    'NMSUBCATEGORIA'  => $row['NMSUBCATEGORIA'],
                    'CDSUBCATEGORIA'  => $row['CDSUBCATEGORIA'],
                    'NRORG'           => $row['NRORG']
                ];
                $this->supUtilities->crud($query, $params);    
        }
        catch (Exception $e) {
             throw $e;
        }
    }
    
    public function updateProduct($row) 
    {
        try {
            $query = "UPDATE PRODSUPPLY SET IDATIVO = :P_IDATIVO
                        WHERE CDCATEGORIA   = :P_CDCATEGORIA
                    AND  CDSUBCATEGORIA = :P_CDSUBCATEGORIA
                    AND  CDFORNECED     = :P_CDFORNECED
                    AND  NRCOLEPRECO = :P_NRCOLEPRECO
                    AND  CDPRODUTO = :P_CDPRODUTO
                    AND  CDMARCA = :P_CDMARCA
                    AND  CDREGIAO = :P_CDREGIAO
                    AND NRORG = :P_NRORG";
                
                    $params = [
                        'P_CDCATEGORIA'       => $row['CDCATEGORIA'],
                        'P_CDSUBCATEGORIA'    => $row['CDSUBCATEGORIA'],
                        'P_CDFORNECED'        => $row['CDFORNECED'],    
                        'P_NRCOLEPRECO'       => $row['NRCOLEPRECO'],
                        'P_CDPRODUTO'         => $row['CDPRODUTO'],
                        'P_CDMARCA'           => $row['CDMARCA'],
                        'P_CDREGIAO'          => $row['CDREGIAO'],
                        'P_IDATIVO'           => $row['IDATIVO'],
                        'P_NRORG'             => $row['NRORG']
                    ];
                $this->supUtilities->crud($query, $params);    
        }
        catch (Exception $e) {
             throw $e;
        }
    }
       
    /********************
     *      DELETES     *
     * ******************/
    
    public function deleteCategory($rows) {
        try {
            $query = "DELETE FROM CATEGORIASUPPLY 
                        WHERE CDCATEGORIA      = :CDCATEGORIA
                         AND NMCATEGORIA        = :NMCATEGORIA
                         AND CDDESPPREV = :CDDESPPREV
                         AND NRORG = :NRORG";
                         
                foreach($rows as $row) {
                    $params = [
                        'CDCATEGORIA'       => $row["CDCATEGORIA"],
                        'NMCATEGORIA'       => $row['NMCATEGORIA'],
                        'CDDESPPREV'        => $row['CDDESPPREV'],
                        'NRORG'             => $row['NRORG']
                    ];
                    $this->supUtilities->crud($query, $params);
                }
        }
        catch (Exception $e) {
             throw $e;
        }
    }
    
    public function deleteSubCategory($row) {
        try {
            $query = "DELETE FROM SUBCATEGSUPPLY 
                        WHERE CDSUBCATEGORIA      = :CDSUBCATEGORIA
                     AND NMSUBCATEGORIA        = :NMSUBCATEGORIA
                     AND CDCATEGORIA = :CDCATEGORIA
                     AND NRORG = :NRORG";
            
                $params = [
                    'CDSUBCATEGORIA'    => $row["CDSUBCATEGORIA"],
                    'NMSUBCATEGORIA'    => $row['NMSUBCATEGORIA'],
                    'CDCATEGORIA'       => $row['CDCATEGORIA'],
                    'NRORG'             => $row['NRORG']
                ];
                $this->supUtilities->crud($query, $params);    
        }
        catch (Exception $e) {
             throw $e;
        }
    }
    
    public function deleteProduct($row) {
        try {
            $query = "DELETE FROM PRODSUPPLY
                        WHERE CDCATEGORIA = :P_CDCATEGORIA
                        AND CDSUBCATEGORIA = :P_CDSUBCATEGORIA
                        AND CDFORNECED = :P_CDFORNECED
                        AND NRCOLEPRECO = :P_NRCOLEPRECO
                        AND CDPRODUTO = :P_CDPRODUTO
                        AND CDMARCA  = :P_CDMARCA
                        AND CDREGIAO  = :P_CDREGIAO
                        AND IDATIVO = :P_IDATIVO
                        AND NRORG = :P_NRORG";
            
            $params = [
                'P_CDCATEGORIA'       => $row['CDCATEGORIA'],
                'P_CDSUBCATEGORIA'    => $row["CDSUBCATEGORIA"],
                'P_CDFORNECED'        => $row['CDFORNECED'],
                'P_NRCOLEPRECO'       => $row['NRCOLEPRECO'],
                'P_CDPRODUTO'         => $row['CDPRODUTO'],
                'P_CDMARCA'           => $row['CDMARCA'],
                'P_CDREGIAO'          => $row['CDREGIAO'],
                'P_IDATIVO'           => $row['IDATIVO'],
                'P_NRORG'             => $row['NRORG']
            ];
            
            $this->supUtilities->crud($query, $params);    
        }
        catch (Exception $e) {
             throw $e;
        }
    }
    
   /********************
    *      SELECTS     *
    * ******************/
    
    public function selectCategory($row) {
        try {
            $query = "SELECT * FROM CATEGORIASUPPLY C,
                                    DESPPREV D
                        WHERE 
                        C.CDDESPPREV = D.CDDESPPREV
                        AND C.CDDESPPREV = :CDDESPPREV
                        AND C.NRORG = :NRORG";
                $params = [
                    'CDDESPPREV'        => $row['CDDESPPREV'],
                    'NRORG'             => $row['NRORG']
                ];
                return $this->supUtilities->crud($query, $params);    
        }
        catch (Exception $e) {
             throw $e;
        }
    }
    
    public function selectCdProduct($row) {
        try {
            $sql = "SELECT DISTINCT
                        P.CDPRODUTO,
                        P.CDARVPROD,
                        P.IDATIVO,
                        P.NMPRODUTO,
                        P.CDNVPRODUTO,
                        P.IDPRODATIVPR,
                        I.CDREGIAO,
                        I.CDMARCA,
                        I.CDFILIAL
                    FROM
                        PRODUTO P,
                        ITEMCOLE I
                    WHERE
                        I.NRCOLEPRECO =:P_NRCOLEPRECO
                        AND P.CDPRODUTO = I.CDPRODUTO
                        AND P.CDARVPROD BETWEEN :P_CDARVPROD_INI AND :P_CDARVPROD_FIN
                        AND P.NRORG = I.NRORG";
                    
            $params = [
                'P_NRCOLEPRECO'     => $row['NRCOLEPRECO'],
                'P_CDARVPROD_INI'   => $row['CDARVPRODINI'],
                'P_CDARVPROD_FIN'   => $row['CDARVPRODFIN'] = self::strPadRigthProd($row['CDARVPRODFIN'])
            ];
            
            return $this->supUtilities->crud($sql, $params);    
        }
        catch (Exception $e) {
             throw $e;
        }
    }
    private function strPadRigthProd($value){
	    return str_pad($value, 20, 'Z', STR_PAD_RIGHT);
	}

    public function checkProductAlreadyRegistered($row) {
        try {
            $sql = "SELECT
                        C.NMCATEGORIA,
                        S.NMSUBCATEGORIA,
                        R.NMREGIAO
                    FROM
                        PRODSUPPLY P,
                        CATEGORIASUPPLY C,
                        SUBCATEGSUPPLY S,
                        REGIAO R
                    WHERE
                        P.CDFORNECED = :P_CDFORNECED
                        AND P.CDPRODUTO = :P_CDPRODUTO
                        AND P.CDMARCA =:P_CDMARCA
                        AND P.CDREGIAO = :P_CDREGIAO
                        AND C.CDCATEGORIA = P.CDCATEGORIA
                        AND S.CDSUBCATEGORIA = P.CDSUBCATEGORIA
                        AND R.CDREGIAO = P.CDREGIAO";
                        
            $params = [
                'P_CDFORNECED' => $row['CDFORNECED'],
                'P_CDPRODUTO' => $row['CDPRODUTO'],
                'P_CDMARCA' => $row['CDMARCA'],
                'P_CDREGIAO' => $row['CDREGIAO']
            ];
            
            $result = $this->supUtilities->crud($sql, $params);
            
            return isset($result[0]) ? $result[0] : $result;
        }
        catch (Exception $e) {
             throw $e;
        }
    }
}