<?php
namespace Teknisa\SUP\Modules\Cotacao\Controllers; 

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SUP\Modules\Cotacao\Services\PedidoColetaService;

class PedidoColetaController extends \Zeedhi\Framework\Controller\Report 
{

    protected $entityManager;
    protected $smtpUtil;
    protected $supUtilities;
    protected $reportService;
    protected $qr2Strategy;
    protected $environment;
    protected $service;
    
	public function __construct($reportService, $qr2Strategy, $entityManager, $smtpUtil, $supUtilities, PedidoColetaService $service, $environment) 
	{
        $this->entityManager = $entityManager;
        $this->smtpUtil = $smtpUtil;
        $this->supUtilities = $supUtilities;
        $this->reportService = $reportService;
        $this->qr2Strategy = $qr2Strategy;
        $this->service = $service;
        $this->environment = $environment;
    }
    
    public function geraPedColetaWeb(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        
        try {
            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'GERAPEDCOLETA_WEB');
            
            $procedure->addParam(new Param('P_IDPEDCOL',        Param::PARAM_INPUT, $row['IDPEDCOL']));
            $procedure->addParam(new Param('P_NRCOTACAO',       Param::PARAM_INPUT, $row['NRCOTACAO']));
            $procedure->addParam(new Param('P_COMPRADOR',       Param::PARAM_INPUT, $row['COMPRADOR']));
            $procedure->addParam(new Param('P_MARCA',           Param::PARAM_INPUT, $row['MARCA']));
            $procedure->addParam(new Param('P_FORNECED',        Param::PARAM_INPUT, $row['FORNECED']));
            $procedure->addParam(new Param('P_REGIAO',          Param::PARAM_INPUT, $row['REGIAO']));
            $procedure->addParam(new Param('P_PRODINI',         Param::PARAM_INPUT, $row['PRODINI']));
            $procedure->addParam(new Param('P_PRODFIN',         Param::PARAM_INPUT, $row['PRODFIN']));
            $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $row['CDFILIAL']));
            $procedure->addParam(new Param('P_CDFILIPROC',      Param::PARAM_INPUT, $row['CDFILIPROC']));
            $procedure->addParam(new Param('P_NRCOMPRA',        Param::PARAM_INPUT, $row['NRCOMPRA']));
            $procedure->addParam(new Param('P_CDLEILAO',        Param::PARAM_INPUT, $row['CDLEILAO']));
            $procedure->addParam(new Param('P_NRSEQPRELIST',    Param::PARAM_INPUT, $row['NRSEQPRELIST']));
            $procedure->addParam(new Param('P_CDOPERLANC',      Param::PARAM_INPUT, $row['CDOPERLANC']));
            $procedure->addParam(new Param('P_DESCFORNECEDOR',  Param::PARAM_INPUT, $row['DESCFORNECEDOR']));
            $procedure->addParam(new Param('P_LISTFORNECEDOR',  Param::PARAM_INPUT, $row['LISTFORNECEDOR']));
            $procedure->addParam(new Param('P_QTMESES',         Param::PARAM_INPUT, $row['QTMESES']));
            $procedure->addParam(new Param('P_DTINI',           Param::PARAM_INPUT, $row['DTINI']));
            $procedure->addParam(new Param('P_DTFIN',           Param::PARAM_INPUT, $row['DTFIN']));
            $procedure->addParam(new Param('P_DTEXPIRAPEDCOL',  Param::PARAM_INPUT, $row['DTEXPIRAPEDCOL']));
            $procedure->addParam(new Param('P_BOPEDEMAIL',      Param::PARAM_INPUT, $row['BOPEDEMAIL']));
            $procedure->addParam(new Param('P_IDPEDGERALREG',   Param::PARAM_INPUT, $row['IDPEDGERALREG']));
            $procedure->addParam(new Param('P_IDPEDGERALFORN',  Param::PARAM_INPUT, $row['IDPEDGERALFORN']));
            $procedure->addParam(new Param('P_IDCURVAABCAF',    Param::PARAM_INPUT, $row['IDCURVAABCAF']));
            $procedure->addParam(new Param('P_IDENVMERCADUM',   Param::PARAM_INPUT, $row['IDENVMERCADUM']));
            $procedure->addParam(new Param('P_BOACRESITEM',     Param::PARAM_INPUT, $row['BOACRESITEM']));
            $procedure->addParam(new Param('P_IDCONSFORNAF',    Param::PARAM_INPUT, $row['IDCONSFORNAF']));
            $procedure->addParam(new Param('P_NRORG',           Param::PARAM_INPUT, $row['NRORG']));
            
            $procedure->execute();
        } catch(Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
        
    }
    
    public function enviaEmailPedColetaWeb(DTO\Request\Row $request, DTO\Response $response) 
    {
        
        $row = $request->getRow();
        
        $this->supUtilities->defineCookieEmail($row['NOME_COOKIE'], $row['MENSAGEM_ORIGINAL']);
        
        $message = "<table>
                            <tr>
                                <td>"
                                    . $row['ASSUNTO'] ." 
                                </td>
                            </tr>
                            
                            <tr>
                                <td class='description'>
                                    <br/>" . $row['MENSAGEM'] .
                                "</td>
                            </tr>
                            <tr class='footer'>
                                <td>
                                    <center>
                                        <br> <p> Email automático enviado pelo Suprimentos. </p> 
                                    </center>
                                </td>
                            </tr>
                        </table>";

        try {
            $messageSendMail = $this->supUtilities->messageSendMail($row['FILIAL_LOGADA'], $row['ASSUNTO'], $message, 'PORTAL E-COMMERCE');
            $this->supUtilities->enviaEmail($row["EMAIL_LOGADO"], $row['ASSUNTO'], $row['EMAILS'], $messageSendMail, $row['EMAILS']);
        } catch(Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function defineCookieEmail(DTO\Request\Row $request, DTO\Response $response) 
    {
        
        $row = $request->getRow();
        
        $this->supUtilities->defineCookieEmail($row['NOME'], $row['VALOR']);
        
    }
    
    public function retornaCookieEmail(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        
        $cookie = $this->supUtilities->retornaCookieEmail($row['NOME']);
        
        $response->addMessage(new DTO\Response\Message($cookie));
    }
    
    public function verificaExistePedidoColeta(DTO\Request\Row $request, DTO\Response $response) 
    {
        
        $row = $request->getRow();
        
        try {
            $query = "SELECT FORNECEDOR.CDFORNECED, FORMATMASK(FORNECEDOR.IDTPIJURFORN,FORNECEDOR.NRINSJURFORN) NRINSJURFORN, FORNECEDOR.NMFANTFORN, REGIAO.CDREGIAO || ' - ' || REGIAO.NMREGIAO NMREGIAO, NRPEDIDOCOL, DTPEDCOLETA, OPERADOR.CDOPERADOR || ' - ' || OPERADOR.NMOPERADOR NMOPERADOR FROM PEDCOLETA, FORNECEDOR, REGIAO, OPERADOR WHERE (( INSTR(:CDREGIAO,PEDCOLETA.CDREGIAO) > 0 ) OR (:CDREGIAO = 'T')) AND (( INSTR(:CDFORNECED,FORNECEDOR.CDFORNECED) > 0 ) OR (:CDFORNECED = 'T')) AND (( INSTR(:CDCOMPRADOR,PEDCOLETA.CDOPERADOR) > 0 ) OR (:CDCOMPRADOR = 'T')) AND ( TRUNC(DTINIPERCOT) <= TRUNC(TO_DATE(:DTFINPERCOT,'DD/MM/YYYY') ) ) AND ( TRUNC(DTFINPERCOT) >= TRUNC(TO_DATE(:DTINIPERCOT,'DD/MM/YYYY') ) ) AND ( PEDCOLETA.CDFORNECED = FORNECEDOR.CDFORNECED ) AND ( PEDCOLETA.CDREGIAO = REGIAO.CDREGIAO ) AND ( PEDCOLETA.CDOPERADOR = OPERADOR.CDOPERADOR ) ORDER BY FORNECEDOR.NRINSJURFORN";
            $params = [
                "CDREGIAO"=> $row['CDREGIAO'],
                "CDFORNECED"=> $row['CDFORNECED'],
                "CDCOMPRADOR"=> $row['CDCOMPRADOR'],
                "DTINIPERCOT"=> $row['DTINIPERCOT'],
                "DTFINPERCOT"=> $row['DTFINPERCOT']
            ];
            
            $existePedido = $this->supUtilities->crud($query, $params);
            
            $response->addMessage(new DTO\Response\Message($existePedido));
        } catch(Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
        
    }
    
    public function verificaOperadorRelacionado(DTO\Request $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        $row['CDOPERADOR'] = $this->environment->getCdoperador();
        $compNaoRelac = [];
        
        if (isset($row['COMPRADOR'])) {
            foreach ($row['COMPRADOR'] as $comprador) {
                $operadoresRelac = $this->service->operadoresRelacionados($comprador);
                if(!self::operadorExistsInArray($operadoresRelac, $row['CDOPERADOR'])){
                    $infoComp = $this->service->getNmComprador($comprador);
                    array_push($compNaoRelac, $infoComp);
                }
                $this->service->getNmComprador($comprador);
            }
        }
       
        $response->addDataSet(new DataSet("data", $compNaoRelac));
    }
    
    public function operadorExistsInArray($operArray, $operadorLog) 
    {
       foreach($operArray as $operador) {
           if($operador['CDOPERADOR'] == $operadorLog) {
               return true;
           }
       }
       return false;
    }
    
    protected $reportMapping = array ();
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $cdfilial = $this->supUtilities->getCentralCompras()['CDFILCENTCMP'];
        $row['P_CDFILIPROC']    = $cdfilial;
        $row['P_CDFILIAL']      = $cdfilial;
        $row['P_CDOPERADOR']    = $this->environment->getCdoperador();
        
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $this->stNomeRel = $this->getReportName($row);
        
        $this->reportMapping[$this->stNomeRel] =
            array (
                'strategy' => $row['STRATEGY'],
                'parameterMapping' => 
                array (
                    "P_CDFILIAL"        => "P_CDFILIAL",
                    "P_NRCOLEPRECO"     => "P_NRCOLEPRECO",
                    "P_NRCOMPRA"        => "P_NRCOMPRA",
                    "P_CDREGIAO"        => "P_CDREGIAO",
                    "P_T_CDREGIAO"      => "P_T_CDREGIAO",
                    "P_CDCOMPRADOR"     => "P_CDCOMPRADOR",
                    "P_T_CDCOMPRADOR"   => "P_T_CDCOMPRADOR",
                    "P_CDFORNECED"      => "P_CDFORNECED",
                    "P_T_CDFORNECED"    => "P_T_CDFORNECED",
                    "P_CDMARCA"         => "P_CDMARCA",
                    "P_T_CDMARCA"       => "P_T_CDMARCA",
                    "P_DTINI"           => "P_DTINI",
                    "P_DTFIN"           => "P_DTFIN",
                    "P_DTINICIAL"       => "P_DTINICIAL",
                    "P_DTFINAL"         => "P_DTFINAL",
                    "P_CDPRODINI"       => "P_CDPRODINI",
                    "P_CDPRODFIN"       => "P_CDPRODFIN",
                    "P_TPRODUTO"        => "P_TPRODUTO",
                    "P_IMPOBS"          => "P_IMPOBS",
                    "P_DESCONSFORN"     => "P_DESCONSFORN",
                    "P_CDFILIPROC"      => "P_CDFILIPROC",
                    "P_ACOMPRAR"        => "P_ACOMPRAR",
                    "P_CDOPERADOR"      => "P_CDOPERADOR"
                )
            );
    }
    
    private function arrayKeyExists($array, $key) 
    {
        return array_key_exists($key , $array) && $array && count($array);
    }
    
    private function concatArray($array, $key)
    {
        return self::arrayKeyExists($array, $key) ? join('_', $array[$key]) : 'T';
    }
    
    private function strPadRightProdFin($array)
    {
        return self::arrayKeyExists($array, 'CDPRODFIN') 
                        ? str_pad($array['CDPRODFIN'], 20, 'Z', STR_PAD_RIGHT) 
                        : str_pad('Z', 20, 'Z', STR_PAD_RIGHT);
    }
    
    private function setReportValues($row)
    {
        $row['P_NRCOLEPRECO']   = $row['NRCOLEPRECO'];
        $row['P_NRCOMPRA']      = '';
        $row['P_CDREGIAO']      = self::concatArray($row, 'REGIAO');
        $row['P_T_CDREGIAO']    = self::concatArray($row, 'REGIAO');
        $row['P_CDCOMPRADOR']   = self::concatArray($row, 'COMPRADOR');
        $row['P_T_CDCOMPRADOR'] = self::concatArray($row, 'COMPRADOR');
        $row['P_CDFORNECED']    = self::concatArray($row, 'FORNECEDOR');
        $row['P_T_CDFORNECED']  = self::concatArray($row, 'FORNECEDOR');
        $row['P_CDMARCA']       = self::concatArray($row, 'MARCA');
        $row['P_T_CDMARCA']     = self::concatArray($row, 'MARCA');
        $row['P_DTINI']         = $row['INTERVAL_VALIDADE']['start'];
        $row['P_DTFIN']         = $row['INTERVAL_VALIDADE']['end'];
        $row['P_DTINICIAL']     = $row['INTERVAL_VALIDADE']['start'];
        $row['P_DTFINAL']       = $row['INTERVAL_VALIDADE']['end'];
        $row['P_CDPRODINI']     = self::arrayKeyExists($row, 'CDPRODINI') ? $row['CDPRODINI'] : '0';
        $row['P_CDPRODFIN']     = self::strPadRightProdFin($row);
        $row['P_TPRODUTO']      = self::arrayKeyExists($row, 'CDPRODINI') ? ' ' : 'T';
        $row['P_IMPOBS']        = 'N';
        $row['P_DESCONSFORN']   = $row['DESCFORNPRI'];
        $row['P_ACOMPRAR']      = $row['TODOSPROD'];
        
        return $row;
    }
    
    public function sendReportToFornecedor(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = self::setReportValues($request->getRow());
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        parent::createRemoteReport(new DTO\Request\Row(new DTO\Row($row), "", "", ""), $response);
        
        var_dump($response);die;
        
        /**
         * 
         * 
         * 
         * 
         * To Do
         * 
         * 
         * 
         * envio de email
         * 
         * SELECT CDFORNECED, NVL(FORNECEDOR.DSMAILPEDCOLCOP, FORNECEDOR.DSMAILCOTCOP) DSMAILCOTCOP,
         * NVL(
         *  (SELECT DSEMAILFORN FROM TABLE(DADOS_FORN_EMAIL(FORNECEDOR.CDFORNECED, NULL, 'S', :NRORG))), 
         *  NVL( FORNECEDOR.DSEMAILPEDCOLFO, FORNECEDOR.DSEMAILCOT )) 
         * AS DSEMAILCOT
         * FROM FORNECEDOR
         * WHERE INSTR(:P_CDFORNECED, FORNECEDOR.CDFORNECED) > 0
         * 
         * 
         * 
        */
        
        $dadosEmail = $rows['email'];
        
        $messages = [];
        
        if (is_array($rows['relatorioData'])) {
            for($i = 0; $i < count($rows['relatorioData']); $i++) {
                try{
                    $row = $rows['relatorioData'][$i];
                    $row['reportLabel'] = 'Relatório';
                    
                    //$this->qr2Strategy->setQr2Type($dadosEmail['TYPE']);
                    //parent::createRemoteReport(new DTO\Request\Row(new DTO\Row($row), "", "", ""), $response);
                    
                    //if(self::verificaParametrosEnvioRelatorioExtra($row)){
                        $row['reportLabel'] = $reportParams['NAME'];
                        $row['__report_name'] = $reportParams['NAME'];
                        $this->birtStrategy->setReportFormat($reportParams['TYPE']);
                        parent::createRemoteReport(new DTO\Request\Row(new DTO\Row($reportFilters), "", "", ""), $response);
                    //}
                    self::enviarEmail($row, $dadosEmail, $response);
                    $this->globalReports = [];
                } 
                catch (Exception $e) {
                    $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), 1));
                }
            }
        }
    }
}
