<?php
namespace Teknisa\SUP\Modules\Cotacao\Modules\PlanilhaCotacao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\SUP\Modules\Cotacao\Modules\PlanilhaCotacao\Services\Sup10700PlanilhaDePrecoObjetivoService;

class Sup10700PlanilhaDePrecoObjetivoController extends \Zeedhi\Framework\Controller\Report {

    protected $entityManager;
    protected $newCode;
    protected $environment;
    
    protected $errors = array();
    protected $errorMessages = array();

    public function __construct($entityManager, $newCode, $environment, $connection, Sup10700PlanilhaDePrecoObjetivoService $service) {
        $this->entityManager = $entityManager;
        $this->newCode       = $newCode;
        $this->connection    = $connection;
        $this->environment   = $environment;
        $this->service       = $service;
    }
    
    public function generateCode($newCode) {
        return $this->newCode->getCodeByTableName($newCode, $this->environment->getNrOrgTrab());
    }
    
    public function parseFloat($floatVar){
        if(is_string($floatVar)){
            $floatVar = str_replace(".", "", $floatVar); 
            $floatVar = str_replace(",", ".", $floatVar);
        }
        return $floatVar;
    }
    
    //LOG DE ERRO
    private function addError($errorType, $forn, $index = 0) {
        if (!isset($this->errors[$index])) {
            $this->errors[$index] = array();
        }
        if (!isset($this->errors[$index][$errorType])) {
            $this->errors[$index][$errorType] = array();
        }
        array_push($this->errors[$index][$errorType], $forn);
    }
    
    
    ////// INÍCIO //////   var_dump($produto);die;
    
    public function insertPrecoObjetivo(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow(); 
            $codFilial = $this->service->getFilial($row);
            
            $dbErro = 0;
            
            if (!empty($codFilial)) {
                foreach ($codFilial as $filial) {
                    $newCode = self::generateCode('PRECOBJETIVO');
        
                    $params = [
                        "P_CDFILIAL"       => $filial['CDFILIAL'],
                        "P_CDREGIAO"       => $filial['CDREGIAO'],
                        "P_TCDFILIAL"      => $row['P_TCDFILIAL'],
                        "P_DTINIVALOBJET"  => $row['P_DTINIVALOBJET'],
                        "P_DTFINVALOBJET"  => $row['P_DTFINVALOBJET'], 
                        "P_PRODINI"        => $row['P_PRODINI'],
                        "P_PRODFIN"        => $row['P_PRODFIN'],
                        "P_TODOSPRODUTOS"  => $row['P_TODOSPRODUTOS'],
                        "P_IDTIPPRODINC"   => $row['P_IDTIPPRODINC'],
                        "P_IDCARGULTPRECO" => $row['P_IDCARGULTPRECO'],
                        "P_NRPRECOBJET"    => $newCode
                    ];
        
                    $this->service->insertPlanObj($params);
                        
                    if ($params["P_IDTIPPRODINC"] == 'N') {
                        $prodSolicita = $this->service->getProdSolicita($params);
                        
                        if (!empty($prodSolicita)) {
                            foreach ($prodSolicita as $solicita) {
                                $params["P_CDREGIAO"]  = $solicita['CDREGIAO'];
                                $params["P_CDPRODUTO"] = $solicita['CDPRODUTO'];
                                
                                if ($row["P_IDCARGULTPRECO"] == 'S') {
                                    $ultPrecoObjetivo = $this->service->getUltPrecoObjetivo($params);
                                    $params["P_VRPRECOBJETIVO"] = $ultPrecoObjetivo['VRPRECOBJETIVO'];
                                    $this->service->insertProdPlanObj($params);
                                }
                            }
                        }
                    } else {
                        $produto = $this->service->getProduto($params);
                        
                        if (!empty($produto)) {
                            foreach ($produto as $codProduto) {
                                $params["P_CDPRODUTO"] = $codProduto['CDPRODUTO'];
                                
                                if ($row["P_IDCARGULTPRECO"] == 'S') {
                                    $ultPrecoObjetivo = $this->service->getUltPrecoObjetivo($params);
                                    $params["P_VRPRECOBJETIVO"] = $ultPrecoObjetivo['VRPRECOBJETIVO'];
                                    $this->service->insertProdPlanObj($params);
                                }
                            }
                        }
                    }

                    $params["P_CDFILIAL"] = $filial['CDFILIAL'];
                    $vdPlanPrecObj = $this->service->getVdPlanPrecObj($params);
                    $countPlanPrecObj = $vdPlanPrecObj['QUANTPRECOBJ'] ? $vdPlanPrecObj['QUANTPRECOBJ'] : 0;
        
                    if ($countPlanPrecObj <= 0) {
                        $params["P_DTPRECOBJET"] = date('d/m/yy');
                        $this->service->deletaPlanilha($params);
                        $dbErro++;
                    }   
                }
            }
            
            $dbErro = ($dbErro > 0) ? 2 : 1;
            $response->addDataSet(new DataSet("resp", [$dbErro]));

        } catch (\Exception $e) {
        	$response->setError(new Error($e->getMessage(), 500));
        }
    }    
    
    public function deletaPlanilha(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $this->service->deletaPlanilha($row);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function deletaProduto(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->service->deletaProduto($row);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
} 